;;; -*-Emacs-Lisp-*-

;; 80aasc-emacs-site.el --- basic config file for AASC Linux systems
;; Copyright (C) 1996-2002 Kurt Hornik <Kurt.Hornik@ci.tuwien.ac.at>
;; Copyright (C) 2006-2007 Kurt Hornik <Kurt.Hornik@wu-wien.ac.at>

;; Last modified: 2007-09-01 by KH

;;;; Basic Customization

(setq-default case-fold-search	nil)
(setq make-backup-files		nil)

;;; load-path
(let* ((site-lisp-directory "/usr/local/share/emacs/site-lisp")
       (user-lisp-directory (expand-file-name "~/lib/emacs/lisp"))
       (files
	(append
 	 (if (file-readable-p site-lisp-directory)
 	     (append (list site-lisp-directory)
 		     (reverse (directory-files
			       site-lisp-directory t "^[^-.]" nil))))
 	 (if (file-readable-p user-lisp-directory)
 	     (append (list user-lisp-directory)
 		     (reverse (directory-files 
 			       user-lisp-directory t "^[^-.]" nil))))
	 )))
  (while files
    (setq file (car files))
    (if (and (or (not file) (file-directory-p file))
 	     (not (member file load-path)))
 	(setq load-path (nconc (list file) load-path)))
    (setq files (cdr files))))

;;; ISO 8859 Latin-1 encoding for European languages and friends
;;; FIXME:
;;; Does not seem to seem to work under Emacs 20?
(if (string-match "^19" emacs-version)
    (progn
      (standard-display-european 1)
      (set-input-mode nil nil 1 7)
      (load "iso-insert")
      (load "iso-cvt")
      ;; iso-insert already sets up C-x8 as a Compose key.  We also
      ;; use C-\ ( la TeX) for that purpose.
      (define-key global-map "\C-\\" 8859-1-map)))

;;; Minor modes
(global-font-lock-mode t)
(transient-mark-mode t)
(add-hook 'c-mode-hook 'turn-on-auto-fill)
(add-hook 'sh-mode-hook 'turn-on-auto-fill)
(add-hook 'awk-mode-hook 'turn-on-auto-fill)
(add-hook 'perl-mode-hook 'turn-on-auto-fill)
(add-hook 'text-mode-hook 'turn-on-auto-fill)
(add-hook 'dired-mode-hook 'turn-on-auto-fill)
(add-hook 'octave-mode-hook 'turn-on-auto-fill)
(add-hook 'emacs-lisp-mode-hook 'turn-on-auto-fill)


;;;; System-specific Customization

;;; GNU/Linux keybindings
(defun RET-behaves-as-LFD ()
  (local-set-key "\C-m" (key-binding "\C-j"))
  (local-set-key "\M-\C-m" (key-binding "\M-\C-j")))

(defun pc-keyboard-setup-hook ()
  (let ((term (or (getenv "TERM") "")))
    (if (string-match "^\(con\|linux\|xterm\|vt100\)" term)
	(progn
	  (define-key function-key-map "\e[1~" [home])
	  (define-key function-key-map "\e[2~" [insert])
	  (define-key function-key-map "\e[3~" [delete])
	  (define-key function-key-map "\e[4~" [end])))))

(defun pc-keybindings-setup ()
  (global-set-key [delete] 'delete-char)
  (global-set-key [C-delete] nil)
  (global-set-key [M-delete] 'kill-word)
  (global-set-key [C-M-delete] 'kill-sexp)
  (global-set-key [C-backspace] nil)
  (global-set-key [M-backspace] 'backward-kill-word)
  (global-set-key [C-M-backspace] 'backward-kill-sexp)
  (global-set-key [C-escape] 'list-buffers)
  (global-set-key [home] 'beginning-of-line)
  (global-set-key [end] 'end-of-line)
  (global-set-key [C-home] 'beginning-of-buffer)
  (global-set-key [C-end] 'end-of-buffer))
  
(if (string-match "lig?nux" system-configuration)
    (progn
      (pc-keybindings-setup)
      (add-hook 'term-setup-hook 'pc-keyboard-setup-hook)
      (add-hook 'sh-mode-hook 'RET-behaves-as-LFD)
      (add-hook 'awk-mode-hook 'RET-behaves-as-LFD)      
      (add-hook 'perl-mode-hook 'RET-behaves-as-LFD)
      (add-hook 'text-mode-hook 'RET-behaves-as-LFD)
      (add-hook 'bibtex-mode-hook 'RET-behaves-as-LFD)
      (add-hook 'octave-mode-hook 'RET-behaves-as-LFD)
      (add-hook 'c-mode-common-hook 'RET-behaves-as-LFD)
      (add-hook 'emacs-lisp-mode-hook 'RET-behaves-as-LFD)
      (add-hook 'lisp-interaction-mode-hook 'RET-behaves-as-LFD)))


;;;; Package Autoloading and Customization

;;; AUCTeX
(setq TeX-parse-self nil
      TeX-auto-save nil
      TeX-auto-local nil)
(add-hook 'TeX-mode-hook
	  (lambda ()
	    (setq LaTeX-item-indent -1)
	    (local-set-key "\"" 'self-insert-command)
	    (add-to-list 'TeX-command-list
			 (list "HyperLaTeX" "hyperlatex %t"
			       'TeX-run-compile nil t))
	    (add-to-list 'TeX-command-list
			 (list "Texinfo" "texi2dvi %t"
			       'TeX-run-TeX nil t))))

;;; bibtex
(setq bibtex-autokey-name-separator "+"
      bibtex-autokey-name-year-separator ":"
      ;; Emacs 22 now has bibtex-autokey-name-case-convert-function
      bibtex-autokey-name-case-convert 'capitalize
      bibtex-autokey-name-case-convert-function 'capitalize
      bibtex-autokey-names 3
      bibtex-autokey-year-length 4
      bibtex-autokey-use-title nil
      bibtex-autokey-titlewords-stretch 0
      bibtex-autokey-titlewords 0)
(add-hook 'bibtex-mode-hook 
	  (lambda ()
	    (setq bibtex-autokey-prefix-string
		  (concat (substring (buffer-name) 0 -4) ":"))))

;;; browser
(setq browse-url-browser-function 'browse-url-netscape)

;;; C
(if (string-match "^20" emacs-version)
    (add-hook 'c-mode-hook
	      (lambda ()
		(c-set-style "bsd")))
  ;; Emacs 21 really ...
  (setq c-default-style "bsd"
	c-basic-offset 4))

;;; ESS
(if (locate-library "ess-site")
    (require 'ess-site))

;;; Gnus
(setq gnus-default-article-saver 'gnus-summary-save-in-file)

;;; html-helper-mode
(if (locate-library "html-helper-mode")
    (lambda ()
      (autoload 'html-helper-mode
	"html-helper-mode" "HTML major mode." t)
      (setq html-helper-use-expert-menu 1
	    html-helper-do-write-file-hooks 1
	    html-helper-timestamp-hook
	    (lambda ()
	      (insert "Last modified: "
		      (format-time-string "%Y-%m-%d %T%z")
		      " by "
		      (user-full-name)
		      "\n")))))

;;; octave
(autoload 'octave-help "octave-hlp" nil t)

;;; perl
(setq perl-indent-level 4)

;;; supercite
(add-hook 'mail-citation-hook 'sc-cite-original)
(setq sc-citation-leader ""
      sc-nested-citation-p t
      sc-auto-fill-region-p nil)

;;; VM
(if (featurep 'vm-init)
    (progn
      (setq vm-folder-directory (expand-file-name "~/Mail/")
	    vm-use-menus 1
	    vm-auto-folder-alist
	    '(("From" 
	       ("[^<]*<\\([^!@]*\\).*" .
		(buffer-substring (match-beginning 1) (match-end 1)))
	       ("\\([^!@ ]*\\)\\($\\|@.*\\| .*\\)" . 
		(buffer-substring (match-beginning 1) (match-end 1)))))
	    ;; This should become unnecessary with VM 6.x
	    vm-url-browser 'vm-mouse-send-url-to-netscape)
      (if window-system
	  (progn
	    (define-key-after menu-bar-tools-menu [rmail] 
	      '("Read Mail" . vm) 'gnus) 
	    (define-key-after menu-bar-tools-menu [smail]
	      '("Send Mail" . vm-mail) 'rmail)))))

;;; W3
(if (featurep 'w3-init)
    (if window-system
	(define-key-after menu-bar-tools-menu [w3] 
	  '("Browse WWW" . w3) 'gnus)))


;;;; Area/site/host/...-specific Customization

;;; Holidays
(defun holiday-easter-date (year)
  "Date of Easter Sunday in YEAR."
  (let* ((century (1+ (/ year 100)))
         (shifted-epact        ;; Age of moon for April 5...
          (% (+ 14 (* 11 (% year 19));;     ...by Nicaean rule
                (-           ;; ...corrected for the Gregorian century rule
                 (/ (* 3 century) 4))
                (/    ;; ...corrected for Metonic cycle inaccuracy.
                 (+ 5 (* 8 century)) 25)
                (* 30 century));;              Keeps value positive.
             30))
         (adjusted-epact       ;;  Adjust for 29.5 day month.
          (if (or (= shifted-epact 0)
                  (and (= shifted-epact 1) (< 10 (% year 19))))
              (1+ shifted-epact)
            shifted-epact))
         (paschal-moon       ;; Day after the full moon on or after March 21.
          (- (calendar-absolute-from-gregorian (list 4 19 year))
             adjusted-epact))
         (abs-easter (calendar-dayname-on-or-before 0 (+ paschal-moon
                                                         7))))
    (calendar-gregorian-from-absolute abs-easter)))
(defun local-holidays-easter-etc ()
  (let* ((abs-easter (calendar-absolute-from-gregorian
		      (holiday-easter-date displayed-year)))
	 (holiday-list
	  (list
	   (list (calendar-gregorian-from-absolute abs-easter)
		 "Ostersonntag")
	   (list (calendar-gregorian-from-absolute (- abs-easter 1))
		 "Karsamstag")
	   (list (calendar-gregorian-from-absolute (- abs-easter 2))
		 "Karfreitag")
	   (list (calendar-gregorian-from-absolute (- abs-easter 3))
		 "Grndonnerstag")
	   (list (calendar-gregorian-from-absolute (- abs-easter 7))
		 "Palmsonntag")
	   (list (calendar-gregorian-from-absolute (- abs-easter 46))
		 "Aschermittwoch")
	   (list (calendar-gregorian-from-absolute (- abs-easter 47))
		 "Faschingsdienstag")
	   (list (calendar-gregorian-from-absolute (- abs-easter 48))
		 "Rosenmontag")
	   (list (calendar-gregorian-from-absolute (+ abs-easter 1))
		 "Ostermontag")
	   (list (calendar-gregorian-from-absolute (+ abs-easter 39))
		 "Christi Himmelfahrt")
	   (list (calendar-gregorian-from-absolute (+ abs-easter 49))
		 "Pfingstsonntag")
	   (list (calendar-gregorian-from-absolute (+ abs-easter 50))
		 "Pfingstmontag")
	   (list (calendar-gregorian-from-absolute (+ abs-easter 60))
		 "Fronleichnam")))
	 (output-list
	  (filter-visible-calendar-holidays holiday-list)))
    output-list))
(setq local-holidays
      (list '(holiday-fixed    1  1 "Neujahr")
	    '(holiday-fixed    1  6 "Heilige Drei Knige")
	    '(holiday-fixed    5  1 "Staatsfeiertag")
	    '(holiday-fixed    8 15 "Maria Himmelfahrt")
	    '(holiday-fixed   10 26 "Nationalfeiertag")
	    '(holiday-fixed   11  1 "Allerheiligen")
	    '(holiday-fixed   12  8 "Maria Empfngnis")
	    '(holiday-fixed   12 25 "Christtag")))
(add-to-list 'local-holidays
	     '(local-holidays-easter-etc))
(add-hook 'calendar-load-hook
	  (lambda ()
	    (add-to-list 'calendar-holidays
			 '(local-holidays-easter-etc))))

